function [fs, pwr] = FFT_power(signal,srate,use_window)
% get frequencies (fs) and power (pwr) from DC to nyquist for input signal 
% and sample rate and boolean to determine if hamming windows is used

% Define FFT parameters
if length(signal) > 20*srate
    fft_points = length(signal);
else
    fft_points = srate*20; % 20-second fft length (better spectral resolution)
end

nSeries = 1;
nPoints = fft_points;

%% Windowing
if use_window
    hamming_window = hamming(length(signal));
    % Apply window
    time_series = signal'.*hamming_window;
else
    time_series = signal';
end

%% Compute FFT, power
% Take fft
FFTX = fft(time_series,fft_points); 
% Calculate the number of unique points 
NumUniquePts = ceil((nPoints+1)/2); 
% FFT is symmetric, throw away second half 
FFTX = FFTX(1:NumUniquePts,:); 
% Take the magnitude of fft of x 
MX = abs(FFTX); 
% Scale the fft so that it is not a function of the length of x 
MX = MX./nPoints; 
% Take the square of the magnitude of fft of x. 
pwr = MX.^2; 
% Multiply by 2 because you threw out second half of FFTX above 
pwr = pwr.*2; 
% DC Component should be unique, i.e. undo multiply by 2. 
pwr(1,:) = pwr(1,:)./2; 
% Nyquist component should also be unique.
if ~rem(nPoints,2) 
   % Here NFFT is even; therefore, Nyquist point is included. 
   pwr(end,:) = pwr(end,:)./2; 
end 

% Compute frequency vector
% This is an evenly spaced frequency vector with NumUniquePts points. 
fs = (0:NumUniquePts-1)* srate / nPoints;
fs=fs';

end